{ OOP-Deklaration des Datentyps OrderedTree                  <OrderedTree.Pas> }
{

Dokumentation der  Klasse TItem
-------------------------------
Die Klasse TItem ist Oberklasse aller Klassen, deren Objekte in einen sortierten
Baum eingefgt werden sollen. Die Ordnungsrelation wird in den Unterklassen von
TItem durch berschreiben der drei abstrakten Methoden isEqual, isGreater und is-
Lower festgelegt.

Dokumentation der Klasse TOrderedTree
-------------------------------------
Objekte der Klasse TOrderedTree verwalten Objekte in einem geordneten Binrbaum
fr den gilt, dass alle Knoteninhalte im linken Unterbaum kleiner sind als der
Wurzelinhalt und alle Knoteninhalte im rechten Teilbaum grer sind als der Wurzelinhalt.
Diese Bedingung gilt auch in allen Unterbumen. Die Knoteninhalte sind Objekte
einer Unterklasse von TItem, in der durch berschreiben der drei Vergleichsmethoden
isLower, isGreater, isEqual (s. TItem) eine eindeutige Ordnungsrelation
festgelegt werden muss.

Konstruktor create
nachher  Der geordnete Binrbaum existiert und ist leer.

Anfrage isEmpty: boolean
nachher  Diese Anfrage liefert den Wahrheitswert true, wenn der geordnete Baum
         leer ist, sonst liefert sie den Wert false.

Auftrag insertItem (pItem: TItem)
nachher  pItem ist entsprechend der Ordnungsrelation in den Baum eingeordnet.

Anfrage searchItem (pItem: TItem):TItem
nachher  Falls ein bezglich der verwendeten Ordnungsrelation mit pItem
         bereinstimmendes Objekt im geordneten Baum enthalten ist, liefert
         die Anfrage dieses, ansonsten wird nil zurckgegeben.

Auftrag deleteItem (pItem: TObject)
nachher  Falls ein bezglich der verwendeten Ordnungsrelation mit pItem
         bereinstimmendes Objekt im Baum enthalten war, wurde dieses entfernt.
         Das Inhaltsobjekt des gelschten Knotens wurde nicht freigegeben.

Anfrage getSortedList: TList
nachher  Die Knoteninhalte des geordneten Binrbaums werden als sortierte Liste
         zurckgegeben. Ist der geordnete Baum leer, wird nil zurckgegeben.

Destruktor destroy
nachher  Der sortierte Binrbaum existiert nicht mehr.

Anfrage Tree: TBinTree
nachher  Die Anfrage liefert den intern gespeicherten Binrbaum vom Typ TBinTree
}
{ ----------------------------------------  Version 2.0 (Garmann) 27.11.2006 - }


unit U_OrderedTree;

interface

uses U_BinTree, U_List;

type
     TItem=class
     public
        function isEqual(pItem: TItem):boolean;virtual;abstract;
        function isLower(pItem: TItem):boolean;virtual;abstract;
        function isGreater(pItem: TItem):boolean;virtual;abstract;
     end;

     TMyOrderedTree=class(TBinTree)
     private
        function IsoliereGroesstesElement(var grElem: TMyOrderedTree): TMyOrderedTree; virtual;
        function EchtLoeschen: TMyOrderedTree; virtual;
     public
        function insertItem(pItem:TItem): TMyOrderedTree;
        function searchItem(pItem:TItem):TItem;
        function deleteItem(pItem:TItem): TMyOrderedTree;
        function getSortedList:TLinList;
     end;

     TOrderedTree=class
     private
        pTree: TMyOrderedTree;
     public
        constructor create;
        function isEmpty: boolean;
        procedure insertItem(pItem:TItem); virtual;
        function searchItem(pItem:TItem):TItem;
        procedure deleteItem(pItem:TItem); virtual;
        function getSortedList:TLinList;
        function Tree: TBinTree;
     end;


implementation


//------------- Methoden der Klasse TMyOrderedTree -------------------------

function TMyOrderedTree.insertItem(pItem:TItem): TMyOrderedTree;
var tb: TMyOrderedTree;
begin
  if isEmpty                                                             { Wenn der Baum leer ist                     }
  then result:= TMyOrderedTree.create(pItem)                               { dann wird das Objekt neue Wurzel           }
  else If TItem(getRootItem).isEqual(pItem)                              { wenn das einzufgende Objekt schon in der Wurzel steht }
       then result:= self                                                { dann lass den Baum unverndert             }
       else if TItem(getRootItem).isGreater(pItem)                       { Wenn die aktuelle Wurzel grer als das einzufgende Objekt ist }
            then begin
                   tb:=TMyOrderedTree(getLeftTree);                        { dann arbeite im linken Teilbaum weiter     }
                   tb:= tb.insertItem(pItem);                            {       Fge es dort ein                     }
                   addTreeLeft(tb);                                      {       hnge den so entstandenen Teilbaum wieder an }
                   result:= self;                                        {       und gib das Gesamtergebenis zurck   }
                 end
            else begin
                   tb:=TMyOrderedTree(getRightTree);                       { sonst arbeite im rechten Teilbaum weiter   }
                   tb:= tb.insertItem(pItem);                            {       Fge es dort ein                     }
                   addTreeRight(tb);                                     {       hnge den so entstandenen Teilbaum wieder an }
                   result:= self;                                        {       und gib das Gesamtergebenis zurck   }
                 end
end;

function TMyOrderedTree.searchItem(pItem:TItem):TItem;
var tb: TMyOrderedTree;
begin
  if isEmpty                                                             { Wenn der Baum leer ist                     }
  then result:= nil                                                      { dann gibt es das gesuchte Objekt nicht     }
  else if TItem(getRootItem).isEqual(pItem)                              { Wenn es gleich der aktuellen Wurzel ist    }
       then result:= TItem(getRootItem)                                  { dann gib die aktuelle Wurzel zurck        }
       else if TItem(getRootItem).isGreater(pItem)                       { wenn die aktuelle Wurzel grer als das gesuchte Objekt ist }
              then result:= TMyOrderedTree(getLeftTree).searchItem(pItem)  { dann suche links weiter                    }
              else result:= TMyOrderedTree(getRightTree).searchItem(pItem);{ sonst suche rechts weiter                  }
end;


    function TMyOrderedTree.IsoliereGroesstesElement(var grElem: TMyOrderedTree): TMyOrderedTree;
    var tb: TMyOrderedTree;
    begin
      if getRightTree = nil
      then begin                                                         { grtes Element gefunden, also:            }
             IsoliereGroesstesElement:= TMyOrderedTree(getLeftTree)  ;     { dieses Element isolieren, d. h. den linken }
                                                                         { Teilbaum zurckliefern.                    }
             grElem:= self;                                              { und das grte Element in CbR-Variablen    }
                                                                         { zurckgeben                                }
           end
      else begin                                                         { noch kein grtes Element                  }
             tb:= TMyOrderedTree(getRightTree);                            { also muss die Suche im rechten Teilbaum    }
             tb:= tb.IsoliereGroesstesElement(grElem);                   { weitergehen. Dieser wird anschlieend ohne }
             addTreeRight(tb);                                           { das grte Element wieder angehngt.       }
             IsoliereGroesstesElement:= self;                            { und der vernderte Baum wird zurckgegeben }
           end;
    end;

  function TMyOrderedTree.EchtLoeschen: TMyOrderedTree;
  var grElement: TMyOrderedTree;                                           { grtes Element im linken Teilbaum         }
      tb: TMyOrderedTree;
  begin
    if getRightTree=nil                                                  { wenn rechter Teilbaum leer ist,            }
    then begin
           result:= TMyOrderedTree(getLeftTree);                           {    dann durch diesen ersetzten.            }
           addTreeLeft(nil);
           free;                                                         {    und aktuelle Wurzel lschen             }
         end
    else if getLeftTree=nil                                              { wenn linker Teilbaum leer ist,             }
         then begin
                result:= TMyOrderedTree(getRightTree);                      {    dann durch diesen ersetzten.            }
                addTreeRight(nil);
                free;                                                    {    und aktuelle Wurzel lschen             }
              end
         else begin
                tb:= TMyOrderedTree(getLeftTree);                          { Linken Teilbaum holen                      }
                tb:= tb.IsoliereGroesstesElement(grElement);             { in diesem das grte Element isolieren     }
                addTreeLeft(tb);                                         { und diesen Teilbaum wieder anhngen        }

                grElement.addTreeLeft(getLeftTree);                      { hnge jetzt aktuellen linken und rechten TB}
                grElement.addTreeRight(getRightTree);                    { an das zuvor ermittelte grte Element an  }
                addTreeLeft(nil);                                        { und trenne die Teilbume vom aktuellen     }
                addTreeRight(nil);                                       { Element.                                   }
                Free;                                                    { Jetzt kann das aktuelle Element gelscht   }
                EchtLoeschen:= grElement;                                { werden und das zuvor grte Element zurck }
              end;                                                       { gegeben werden.                            }
  end;

function TMyOrderedTree.deleteItem(pItem:TItem): TMyOrderedTree;
var tb: TMyOrderedTree;
begin
  If isEmpty                                                             { Wenn Baum leer ist.                        }
  then result:= Self                                                     { Gib den unvernderten Baum zurck          }
  else begin
         if TItem(getRootItem).isEqual(pItem)                            { Wenn das Suchelement existiert, dann       }
         then result:= EchtLoeschen                                      { aktuellen Baum ohne das Element e zurck-  }
         else if TItem(getRootItem).isGreater(pItem)                     { Wenn die aktuelle Wurzel grer als e, dann}
              then begin
                     tb:= TMyOrderedTree(getLeftTree);                     { Suche im linken Teilbaum weiter und lsche }
                     tb:= tb.deleteItem(pItem);                          { darin das Element.                         }
                     addTreeLeft(tb);                                    { Hnge anschlieend den vernderten Baum    }
                     result:= Self;                                      { links an und gib den gesamten Baum zurck. }
                   end
              else begin
                     tb:= TMyOrderedTree(getRightTree);                    { Suche im rechten Teilbaum weiter und lsche}
                     tb:= tb.deleteItem(pItem);                          { darin das Element.                         }
                     addTreeRight(tb);                                   { Hnge anschlieend den vernderten Baum    }
                     result:= Self;                                      { rechts an und gib den gesamten Baum zurck.}
                   end;
       end;
end;

function TMyOrderedTree.getSortedList:TLinList;
var List: TLinList;

  procedure InorderAusgabe(pTree: TBinTree; pList: TLinList);
  begin
    if not pTree.isEmpty
    then begin
           InorderAusgabe(pTree.getLeftTree,pList);
           pList.insertBehind(pTree.getRootItem);
           pList.next;
           InorderAusgabe(pTree.getRightTree,pList);
         end;
  end;

begin
  if isEmpty
  then result:= nil
  else begin
         List:= TLinList.create;
         InorderAusgabe(self,List);
         result:= List;
       end;
end;




//------------- Methoden der Klasse TOrderedTree -------------------------

constructor TOrderedTree.create;
begin
  pTree:= nil;
end;

function TOrderedTree.isEmpty: boolean;
begin
  result:= pTree.isEmpty;
end;

procedure TOrderedTree.insertItem(pItem:TItem);
begin
  pTree:= pTree.insertItem(pItem);
end;

function TOrderedTree.searchItem(pItem:TItem):TItem;
begin
  result:= pTree.searchItem(pItem);
end;

procedure TOrderedTree.deleteItem(pItem:TItem);
begin
  pTree:= pTree.deleteItem(pItem)
end;

function TOrderedTree.getSortedList:TLinList;
begin
  result:= pTree.getSortedList;
end;

function TOrderedTree.Tree: TBinTree;
begin
  result:= pTree;
end;


end.
